@@ -13,7 +13,6 @@
 
 import ast
 import bdb
-import cmd
 import io
 import logging
 import os
@@ -23,17 +22,15 @@ import sys
 import time
 import warnings
 
-from IPython import __version__ as ipy_version
 from IPython.core.getipython import get_ipython
 
-from spyder_kernels.comms.frontendcomm import CommError, frontend_request
+from spyder_kernels.comms.frontendcomm import frontend_request
 from spyder_kernels.customize.namespace_manager import NamespaceManager
 from spyder_kernels.customize.spyderpdb import SpyderPdb, get_new_debugger
 from spyder_kernels.customize.umr import UserModuleReloader
 from spyder_kernels.py3compat import (
-    TimeoutError, PY2, _print, encode, compat_exec, FileNotFoundError)
-from spyder_kernels.customize.utils import (
-    capture_last_Expr, normalise_filename)
+    PY2, _print, encode, compat_exec, FileNotFoundError)
+from spyder_kernels.customize.utils import capture_last_Expr, canonic
 
 if not PY2:
     from IPython.core.inputtransformer2 import (
@@ -330,18 +327,8 @@ os.get_terminal_size = _patched_get_terminal_size
 # =============================================================================
 # Pdb adjustments
 # =============================================================================
-def cmd_input(prompt=''):
-    return get_ipython().kernel.cmd_input(prompt)
-
-
 pdb.Pdb = SpyderPdb
 
-if PY2:
-    cmd.raw_input = cmd_input
-else:
-    cmd.input = cmd_input
-
-
 # =============================================================================
 # User module reloader
 # =============================================================================
@@ -366,8 +353,12 @@ def post_mortem_excepthook(type, value, tb):
         _print('*' * 40)
         _print('Entering post mortem debugging...')
         _print('*' * 40)
-        #  add ability to move between frames
-        p.send_initial_notification = False
+
+        # Inform Spyder about position of exception: pdb.Pdb.interaction() calls
+        # cmd.Cmd.cmdloop(), which calls SpyderPdb.preloop() where
+        # send_initial_notification is handled.
+        p.send_initial_notification = True
+
         p.reset()
         frame = tb.tb_next.tb_frame
         # wait for stdout to print
@@ -471,7 +462,7 @@ def exec_code(code, filename, ns_globals, ns_locals=None, post_mortem=False,
         else:
             ast_code = ast.parse(transform_cell(code))
 
-        if code.rstrip()[-1] == ";":
+        if code.rstrip()[-1:] == ";":
             # Supress output with ;
             capture_last_expression = False
 
@@ -505,18 +496,23 @@ def exec_code(code, filename, ns_globals, ns_locals=None, post_mortem=False,
         __tracebackhide__ = "__pdb_exit__"
 
 
-def get_file_code(filename, save_all=True):
-    """Retrive the content of a file."""
+def get_file_code(filename, save_all=True, raise_exception=False):
+    """Retrieve the content of a file."""
     # Get code from spyder
     try:
-        file_code = frontend_request(blocking=True).get_file_code(
+        return frontend_request(blocking=True).get_file_code(
             filename, save_all=save_all)
-    except (CommError, TimeoutError, RuntimeError, FileNotFoundError):
-        file_code = None
-    if file_code is None:
-        with open(filename, 'r') as f:
-            return f.read()
-    return file_code
+    except Exception:
+        # Maybe this is a local file
+        try:
+            with open(filename, 'r') as f:
+                return f.read()
+        except FileNotFoundError:
+            pass
+        if raise_exception:
+            raise
+        # Else return None
+        return None
 
 
 def runfile(filename=None, args=None, wdir=None, namespace=None,
@@ -536,7 +532,7 @@ def runfile(filename=None, args=None, wdir=None, namespace=None,
 
 def _exec_file(filename=None, args=None, wdir=None, namespace=None,
                post_mortem=False, current_namespace=False, stack_depth=0,
-               exec_fun=None):
+               exec_fun=None, canonic_filename=None):
     # Tell IPython to hide this frame (>7.16)
     __tracebackhide__ = True
     ipython_shell = get_ipython()
@@ -544,11 +540,6 @@ def _exec_file(filename=None, args=None, wdir=None, namespace=None,
         filename = get_current_file_name()
         if filename is None:
             return
-    else:
-        # get_debugger replaces \\ by / so we must undo that here
-        # Otherwise code caching doesn't work
-        if os.name == 'nt':
-            filename = filename.replace('/', '\\')
 
     try:
         filename = filename.decode('utf-8')
@@ -562,22 +553,23 @@ def _exec_file(filename=None, args=None, wdir=None, namespace=None,
         __umr__.run()
     if args is not None and not isinstance(args, basestring):
         raise TypeError("expected a character buffer object")
+
     try:
-        file_code = get_file_code(filename)
+        file_code = get_file_code(filename, raise_exception=True)
     except Exception:
+        # Show an error and return None
         _print(
             "This command failed to be executed because an error occurred"
             " while trying to get the file code from Spyder's"
             " editor. The error was:\n\n")
         get_ipython().showtraceback(exception_only=True)
         return
-    if file_code is None:
-        _print("Could not get code from editor.\n")
-        return
 
-    # Normalise the filename
-    filename = os.path.abspath(filename)
-    filename = os.path.normcase(filename)
+    # Here the remote filename has been used. It must now be valid locally.
+    if canonic_filename is not None:
+        filename = canonic_filename
+    else:
+        filename = canonic(filename)
 
     with NamespaceManager(filename, namespace, current_namespace,
                           file_code=file_code, stack_depth=stack_depth + 1
@@ -655,7 +647,7 @@ def debugfile(filename=None, args=None, wdir=None, post_mortem=False,
     if shell.is_debugging():
         # Recursive
         code = (
-            "runfile({}".format(repr(normalise_filename(filename))) +
+            "runfile({}".format(repr(filename)) +
             ", args=%r, wdir=%r, current_namespace=%r)" % (
                 args, wdir, current_namespace)
         )
@@ -666,11 +658,13 @@ def debugfile(filename=None, args=None, wdir=None, post_mortem=False,
     else:
         debugger = get_new_debugger(filename, True)
         _exec_file(
-            filename=debugger.canonic(filename),
-            args=args, wdir=wdir,
+            filename=filename,
+            canonic_filename=debugger.canonic(filename),
+            args=args,
+            wdir=wdir,
             current_namespace=current_namespace,
             exec_fun=debugger.run,
-            stack_depth=1
+            stack_depth=1,
         )
 
 
@@ -696,7 +690,7 @@ def runcell(cellname, filename=None, post_mortem=False):
 
 
 def _exec_cell(cellname, filename=None, post_mortem=False, stack_depth=0,
-               exec_fun=None):
+               exec_fun=None, canonic_filename=None):
     """
     Execute a code cell with a given exec function.
     """
@@ -706,11 +700,6 @@ def _exec_cell(cellname, filename=None, post_mortem=False, stack_depth=0,
         filename = get_current_file_name()
         if filename is None:
             return
-    else:
-        # get_debugger replaces \\ by / so we must undo that here
-        # Otherwise code caching doesn't work
-        if os.name == 'nt':
-            filename = filename.replace('/', '\\')
     try:
         filename = filename.decode('utf-8')
     except (UnicodeError, TypeError, AttributeError):
@@ -736,14 +725,14 @@ def _exec_cell(cellname, filename=None, post_mortem=False, stack_depth=0,
     # Trigger `post_execute` to exit the additional pre-execution.
     # See Spyder PR #7310.
     ipython_shell.events.trigger('post_execute')
-    try:
-        file_code = get_file_code(filename, save_all=False)
-    except Exception:
-        file_code = None
+    file_code = get_file_code(filename, save_all=False)
 
-    # Normalise the filename
-    filename = os.path.abspath(filename)
-    filename = os.path.normcase(filename)
+    # Here the remote filename has been used. It must now be valid locally.
+    if canonic_filename is not None:
+        filename = canonic_filename
+    else:
+        # Normalise the filename
+        filename = canonic(filename)
 
     with NamespaceManager(filename, current_namespace=True,
                           file_code=file_code, stack_depth=stack_depth + 1
@@ -770,7 +759,7 @@ def debugcell(cellname, filename=None, post_mortem=False):
         # Recursive
         code = (
             "runcell({}, ".format(repr(cellname)) +
-            "{})".format(repr(normalise_filename(filename)))
+            "{})".format(repr(filename))
         )
         shell.pdb_session.enter_recursive_debugger(
             code, filename, False,
@@ -779,7 +768,8 @@ def debugcell(cellname, filename=None, post_mortem=False):
         debugger = get_new_debugger(filename, False)
         _exec_cell(
             cellname=cellname,
-            filename=debugger.canonic(filename),
+            filename=filename,
+            canonic_filename=debugger.canonic(filename),
             exec_fun=debugger.run,
             stack_depth=1
         )
@@ -814,12 +804,15 @@ builtins.cell_count = cell_count
 
 
 # =============================================================================
-# Extend sys.path with paths that come from Spyder
+# PYTHONPATH and sys.path Adjustments
 # =============================================================================
+# PYTHONPATH is not passed to kernel directly, see spyder-ide/spyder#13519
+# This allows the kernel to start without crashing if modules in PYTHONPATH
+# shadow standard library modules.
 def set_spyder_pythonpath():
     pypath = os.environ.get('SPY_PYTHONPATH')
     if pypath:
-        pathlist = pypath.split(os.pathsep)
-        sys.path.extend(pathlist)
+        sys.path.extend(pypath.split(os.pathsep))
+        os.environ.update({'PYTHONPATH': pypath})
 
 set_spyder_pythonpath()
